/* eslint-disable @typescript-eslint/naming-convention */ import * as React from 'react';
import { decodeBase64Fields, isArrayOrTypedArray, isDateArray, isMonthArray, isNumberArray, isYearArray, mapFluentChart, sanitizeJson } from '@fluentui/chart-utilities';
import { tokens } from '@fluentui/react-theme';
import { ThemeContext_unstable as V9ThemeContext } from '@fluentui/react-shared-contexts';
import { webLightTheme } from '@fluentui/tokens';
import * as d3Color from 'd3-color';
import { correctYearMonth, transformPlotlyJsonToDonutProps, transformPlotlyJsonToVSBCProps, transformPlotlyJsonToScatterChartProps, transformPlotlyJsonToHorizontalBarWithAxisProps, transformPlotlyJsonToHeatmapProps, transformPlotlyJsonToSankeyProps, transformPlotlyJsonToGaugeProps, transformPlotlyJsonToGVBCProps, transformPlotlyJsonToVBCProps } from './PlotlySchemaAdapter';
import { DonutChart } from '../DonutChart/index';
import { VerticalStackedBarChart } from '../VerticalStackedBarChart/index';
import { LineChart } from '../LineChart/index';
import { HorizontalBarChartWithAxis } from '../HorizontalBarChartWithAxis/index';
import { AreaChart } from '../AreaChart/index';
import { HeatMapChart } from '../HeatMapChart/index';
import { SankeyChart } from '../SankeyChart/SankeyChart';
import { GaugeChart } from '../GaugeChart/index';
import { GroupedVerticalBarChart } from '../GroupedVerticalBarChart/index';
import { VerticalBarChart } from '../VerticalBarChart/index';
import { toImage } from './imageExporter';
import { ScatterChart } from '../ScatterChart/index';
import { withResponsiveContainer } from '../ResponsiveContainer/withResponsiveContainer';
const ResponsiveDonutChart = withResponsiveContainer(DonutChart);
const ResponsiveVerticalStackedBarChart = withResponsiveContainer(VerticalStackedBarChart);
const ResponsiveLineChart = withResponsiveContainer(LineChart);
const ResponsiveHorizontalBarChartWithAxis = withResponsiveContainer(HorizontalBarChartWithAxis);
const ResponsiveAreaChart = withResponsiveContainer(AreaChart);
const ResponsiveHeatMapChart = withResponsiveContainer(HeatMapChart);
const ResponsiveSankeyChart = withResponsiveContainer(SankeyChart);
const ResponsiveGaugeChart = withResponsiveContainer(GaugeChart);
const ResponsiveGroupedVerticalBarChart = withResponsiveContainer(GroupedVerticalBarChart);
const ResponsiveVerticalBarChart = withResponsiveContainer(VerticalBarChart);
const ResponsiveScatterChart = withResponsiveContainer(ScatterChart);
const useColorMapping = ()=>{
    const colorMap = React.useRef(new Map());
    return colorMap;
};
const useIsDarkTheme = ()=>{
    const parentV9Theme = React.useContext(V9ThemeContext);
    const v9Theme = parentV9Theme ? parentV9Theme : webLightTheme;
    // Get background and foreground colors
    const backgroundColor = d3Color.hsl(v9Theme.colorNeutralBackground1);
    const foregroundColor = d3Color.hsl(v9Theme.colorNeutralForeground1);
    const isDarkTheme = backgroundColor.l < foregroundColor.l;
    return isDarkTheme;
};
/**
 * DeclarativeChart component.
 * {@docCategory DeclarativeChart}
 */ export const DeclarativeChart = /*#__PURE__*/ React.forwardRef((props, forwardedRef)=>{
    const { plotlySchema } = sanitizeJson(props.chartSchema);
    const chart = mapFluentChart(plotlySchema);
    if (!chart.isValid) {
        throw new Error(`Invalid chart schema: ${chart.errorMessage}`);
    }
    let plotlyInput = plotlySchema;
    try {
        plotlyInput = decodeBase64Fields(plotlyInput);
    } catch (error) {
        throw new Error(`Failed to decode plotly schema: ${error}`);
    }
    const plotlyInputWithValidData = {
        ...plotlyInput,
        data: chart.validTracesInfo.map((trace)=>plotlyInput.data[trace.index])
    };
    let { selectedLegends } = plotlySchema;
    const colorMap = useColorMapping();
    const isDarkTheme = useIsDarkTheme();
    const chartRef = React.useRef(null);
    if (!isArrayOrTypedArray(selectedLegends)) {
        selectedLegends = [];
    }
    const [activeLegends, setActiveLegends] = React.useState(selectedLegends);
    const onActiveLegendsChange = (keys)=>{
        setActiveLegends(keys);
        if (props.onSchemaChange) {
            props.onSchemaChange({
                plotlySchema: {
                    plotlyInput,
                    selectedLegends: keys
                }
            });
        }
    };
    React.useEffect(()=>{
        // eslint-disable-next-line @typescript-eslint/no-shadow
        const { plotlySchema } = sanitizeJson(props.chartSchema);
        // eslint-disable-next-line @typescript-eslint/no-shadow
        const { selectedLegends } = plotlySchema;
        setActiveLegends(selectedLegends !== null && selectedLegends !== void 0 ? selectedLegends : []);
    }, [
        props.chartSchema
    ]);
    const multiSelectLegendProps = {
        canSelectMultipleLegends: true,
        onChange: onActiveLegendsChange,
        selectedLegends: activeLegends
    };
    const commonProps = {
        legendProps: multiSelectLegendProps,
        componentRef: chartRef
    };
    const renderLineAreaScatter = (plotlyData, isAreaChart)=>{
        var _plotlyData_;
        const isScatterMarkers = ((_plotlyData_ = plotlyData[0]) === null || _plotlyData_ === void 0 ? void 0 : _plotlyData_.mode) === 'markers';
        const chartProps = {
            ...transformPlotlyJsonToScatterChartProps({
                data: plotlyData,
                layout: plotlyInput.layout
            }, isAreaChart, colorMap, isDarkTheme),
            ...commonProps
        };
        if (isAreaChart) {
            return /*#__PURE__*/ React.createElement(ResponsiveAreaChart, chartProps);
        }
        if (isScatterMarkers) {
            return /*#__PURE__*/ React.createElement(ResponsiveScatterChart, chartProps);
        }
        return /*#__PURE__*/ React.createElement(ResponsiveLineChart, chartProps);
    };
    const checkAndRenderChart = (isAreaChart = false)=>{
        let fallbackVSBC = false;
        const xValues = plotlyInputWithValidData.data[0].x;
        const isXDate = isDateArray(xValues);
        const isXNumber = isNumberArray(xValues);
        const isXMonth = isMonthArray(xValues);
        // Consider year as categorical variable not numeric continuous variable
        // Also year is not considered a date variable as it is represented as a point
        // in time and brings additional complexity of handling timezone and locale
        // formatting given the current design of the charting library
        const isXYear = isYearArray(xValues);
        if ((isXDate || isXNumber) && !isXYear) {
            return renderLineAreaScatter(plotlyInputWithValidData.data, isAreaChart);
        } else if (isXMonth) {
            const updatedData = plotlyInputWithValidData.data.map((dataPoint)=>({
                    ...dataPoint,
                    x: correctYearMonth(dataPoint.x)
                }));
            return renderLineAreaScatter(updatedData, isAreaChart);
        }
        // Unsupported schema, render as VerticalStackedBarChart
        fallbackVSBC = true;
        return /*#__PURE__*/ React.createElement(ResponsiveVerticalStackedBarChart, {
            ...transformPlotlyJsonToVSBCProps(plotlyInputWithValidData, colorMap, isDarkTheme, fallbackVSBC),
            ...commonProps
        });
    };
    // TODO
    const exportAsImage = React.useCallback((opts)=>{
        var _chartRef_current;
        return toImage((_chartRef_current = chartRef.current) === null || _chartRef_current === void 0 ? void 0 : _chartRef_current.chartContainer, {
            background: tokens.colorNeutralBackground1,
            scale: 5,
            ...opts
        });
    }, []);
    React.useImperativeHandle(props.componentRef, ()=>({
            exportAsImage
        }), [
        exportAsImage
    ]);
    switch(chart.type){
        case 'donut':
            return /*#__PURE__*/ React.createElement(ResponsiveDonutChart, {
                ...transformPlotlyJsonToDonutProps(plotlyInputWithValidData, colorMap, isDarkTheme),
                ...commonProps
            });
        case 'horizontalbar':
            return /*#__PURE__*/ React.createElement(ResponsiveHorizontalBarChartWithAxis, {
                ...transformPlotlyJsonToHorizontalBarWithAxisProps(plotlyInputWithValidData, colorMap, isDarkTheme),
                ...commonProps
            });
        case 'groupedverticalbar':
            return /*#__PURE__*/ React.createElement(ResponsiveGroupedVerticalBarChart, {
                ...transformPlotlyJsonToGVBCProps(plotlyInputWithValidData, colorMap, isDarkTheme),
                ...commonProps
            });
        case 'verticalstackedbar':
            return /*#__PURE__*/ React.createElement(ResponsiveVerticalStackedBarChart, {
                ...transformPlotlyJsonToVSBCProps(plotlyInputWithValidData, colorMap, isDarkTheme),
                ...commonProps
            });
        case 'heatmap':
            return /*#__PURE__*/ React.createElement(ResponsiveHeatMapChart, {
                ...transformPlotlyJsonToHeatmapProps(plotlyInputWithValidData),
                ...commonProps,
                legendProps: {}
            });
        case 'sankey':
            return /*#__PURE__*/ React.createElement(ResponsiveSankeyChart, {
                ...transformPlotlyJsonToSankeyProps(plotlyInputWithValidData, colorMap, isDarkTheme),
                ...commonProps
            });
        case 'gauge':
            return /*#__PURE__*/ React.createElement(ResponsiveGaugeChart, {
                ...transformPlotlyJsonToGaugeProps(plotlyInputWithValidData, colorMap, isDarkTheme),
                ...commonProps
            });
        case 'verticalbar':
            return /*#__PURE__*/ React.createElement(ResponsiveVerticalBarChart, {
                ...transformPlotlyJsonToVBCProps(plotlyInputWithValidData, colorMap, isDarkTheme),
                ...commonProps
            });
        // TODO: Add 'scatter' as a separate chart type
        case 'area':
        case 'line':
        case 'fallback':
            // Need recheck for area chart as we don't have ability to check for valid months in previous step
            const isAreaChart = plotlyInputWithValidData.data.some((series)=>series.fill === 'tonexty' || series.fill === 'tozeroy' || !!series.stackgroup);
            return checkAndRenderChart(isAreaChart);
        default:
            var _plotlyInputWithValidData_data_;
            throw new Error(`Unsupported chart type :${(_plotlyInputWithValidData_data_ = plotlyInputWithValidData.data[0]) === null || _plotlyInputWithValidData_data_ === void 0 ? void 0 : _plotlyInputWithValidData_data_.type}`);
    }
});
DeclarativeChart.displayName = 'DeclarativeChart';
